﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.Audio;

public abstract class SoundClass : MonoBehaviour {

	[System.Serializable]
	protected struct soundGroup{
		public string soundName;
		public AudioClip[] soundArray;
		public AudioMixerGroup mixer;
	}
		
	public enum SoundTypeEnum{
		oneShot,
		loop
	}

	[SerializeField] protected AudioSource[] loopSources;
	[SerializeField] protected soundGroup[] sounds;


	protected virtual void Start(){
		SetReferences ();
	}

	protected virtual void SetReferences(){

	}

	/// <summary>
	/// Plays a Random Sound from an Array.
	/// </summary>
	/// <param name="audio">Audio.</param>
	/// <param name="source">Source.</param>
	protected void PlayRandomInArray (AudioClip[] audio,AudioSource source){
		int chosenSound = Random.Range (0, audio.Length - 1);
		source.PlayOneShot (audio [chosenSound]);
	}
		
	/// <summary>
	/// Plays a sound.
	/// </summary>
	/// <param name="soundNum">Sound number.</param>
	/// <param name="stopAllPreviousSounds">If set to <c>true</c> stop all previous sounds.</param>
	/// <param name="soundType">Sound type.</param>
	/// <param name="source">Source.</param>
	/// <param name="soundArray">Sound array.</param>
	/// <param name="includeLoopsInStop">If set to <c>true</c> include loops in stop.</param>
	public void PlaySound(int soundNum, bool stopAllPreviousSounds = false,
		SoundTypeEnum soundType = SoundTypeEnum.loop, AudioSource source = null, AudioClip[] soundArray = null, bool includeLoopsInStop = false)
	{
		if (soundNum < 0) {
			Debug.LogError ("Invalid Sound Number logged: SoundClass, " + gameObject.name);
		}

		switch (soundType) {
		case (SoundTypeEnum.loop):
			
			if (stopAllPreviousSounds) {
				StopAllLoops ();
			}
			if (!loopSources [soundNum].enabled) {
				loopSources [soundNum].enabled = true;
				loopSources [soundNum].loop = true;
			}
			break;
		case (SoundTypeEnum.oneShot):
			if (stopAllPreviousSounds) {
				source.Stop ();
				if (includeLoopsInStop) {
					StopAllLoops ();
				}
			}
			source.loop = false;
			source.PlayOneShot (soundArray [soundNum]);
			break;
		default:
			break;
		}
	

	}

	protected void StopAllLoops(){
		foreach (AudioSource sour in loopSources) {
			sour.enabled = false;
		}
	}

	protected void StopSound(AudioSource source, AudioClip[] soundArray = null){
		if (soundArray != null) {
			foreach (AudioClip sound in soundArray) {
				if (source.isPlaying) {
					if (source.clip == sound) {
						source.Stop ();
					}
				}
			}
		} else if (source.isPlaying) {
			source.Stop ();
		} 
	}

}
